from .types import (
    TextFileMode
)
from .constants import (
    TEMPLATES,
    MSGID_REGEX
)


class POTFile:
    def __init__(self, file: str, mode: TextFileMode, template: str | None = None, encoding: str | None = None):
        self._path = file
        self._mode = mode
        self._template = template
        self._file = open(self._path, mode=mode, encoding=encoding)
        self._cached_keys = set()

        if self._mode == 'w':
            self._write_template()

        if self._mode in {'a', 'r'}:
            self._load_keys()

    def _write_template(self):
        if self._template:
            self._file.write(self._template)

    def _load_keys(self):
        for line in self._file:
            if MSGID_REGEX.match(line):
                result = MSGID_REGEX.findall(line)[0]
                self._cached_keys.add(result)

    def write(self, key: str, comment: str = None, check_key: bool = True):
        if check_key and key in self._cached_keys:
            raise KeyError(f'Duplicate key: {key}')

        self._cached_keys.add(key)
        text = f'msgid "{key}"\nmsgstr ""\n\n'
        if comment:
            text = f'# {comment}\n{text}'
        self._file.write(text)

    @property
    def keys(self):
        yield from self

    def close(self):
        self._file.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self._file.close()

    def __iter__(self):
        yield from self._cached_keys



import gettext

from i18n.types import BaseTranslatableEnum
from i18n.utils.gettext import (
    language,
    get_language
)


class TranslatableEnum(BaseTranslatableEnum):
    @property
    def value(self) -> str:
        value = super(TranslatableEnum, self).value
        return gettext.gettext(value)

    def language(self, lang: str) -> str:
        with language(lang=lang):
            return self.value

    def __str__(self):
        return self.value



from i18n.types import BaseTranslatableEnum
from i18n.utils.clsutils import iter_subclasses


def extract_keys(base_enum: type[BaseTranslatableEnum] = BaseTranslatableEnum) -> set[str]:
    keys = set()
    for subenum in iter_subclasses(base_enum):
        for item in subenum:
            keys.add(item.value)
    return keys



from contextlib import contextmanager

from .shortcuts import (
    get_language,
    set_language
)


@contextmanager
def language(lang: str):
    actual_lang = get_language()

    try:
        set_language(lang)
        yield
    finally:
        set_language(actual_lang)



import argparse
import sys
from typing import Iterable

from i18n.settings import (
    APP,
    DESCRIPTION,
    VERSION
)
from i18n.types import (
    BaseTranslatableEnum
)
from i18n.tools import (
    extract_keys
)
from i18n.utils.gettext import (
    set_language
)
from i18n.utils.potfile import (
    POTFile,
    TEMPLATES
)
from i18n.utils.imputils import (
    import_module
)


def setup_argparse() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog=APP,
        description=DESCRIPTION
    )

    parser.add_argument(
        'source',
        default='main.py',
        help='source file to extract translations from (main file of application)',
        nargs='?'
    )

    parser.add_argument(
        'destination',
        default='main.pot',
        help='destination .pot file',
        nargs='?'
    )

    parser.add_argument(
        '-v', '--version',
        action='store_true',
        help='show version and exit',
        default=False
    )

    parser.add_argument(
        '-t', '--template',
        required=False,
        default='common',
        help='.pot file template',
        choices=set(TEMPLATES.keys())
    )

    return parser


def show_version():
    print(VERSION)


def save_pot(file: str, keys: Iterable[str], template: str = None) -> None:
    with POTFile(file=file, mode='w', template=template) as potfile:
        for key in keys:
            potfile.write(key)


def main():
    parser = setup_argparse()
    args = parser.parse_args()

    if args.version:
        show_version()
        sys.exit(0)

    source = args.source
    try:
        import_module(source)
    except ImportError:
        raise

    set_language('en')

    keys = extract_keys(base_enum=BaseTranslatableEnum)
    save_pot(
        file=args.destination,
        keys=keys,
        template=TEMPLATES[args.template]
    )


if __name__ == '__main__':
    main()



APP = 'Translatable-Enums'
DESCRIPTION = ('Translatable-Enums is a i18n tool which uses built-in Enums '
               'as an convenient way to store translation keys.')
VERSION = '0.0.2'
AUTHOR = 'crazyproger1'



from .shortcuts import (
    set_language,
    set_domain,
    get_language
)

from .contexts import (
    language
)

__all__ = [
    'set_domain',
    'set_language',
    'get_language',
    'language'
]



import gettext
import os


def set_language(language: str):
    assert isinstance(language, str), 'Language must be string'

    os.environ['LANG'] = language


def get_language() -> str:
    return os.environ.get('LANG')


def set_domain(domain: str, localedir: str):
    gettext.bindtextdomain(
        domain=domain,
        localedir=localedir
    )
    gettext.textdomain(domain=domain)



import os
import sys
import importlib.util
from functools import cache
from types import ModuleType


@cache
def import_module(path: str) -> ModuleType:
    if os.path.isdir(path):
        path = os.path.join(path, '__main__.py')

    directory = os.path.dirname(path)
    sys.path.append(directory)
    try:
        if not os.path.exists(path):
            raise
        *_, filename = os.path.split(path)
        spec = importlib.util.spec_from_file_location(filename.replace('.py', ''), path)
        imported_module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(imported_module)
        return imported_module
    except Exception:
        raise ImportError(f'Failed to import module: {path}')
    finally:
        sys.path.remove(directory)



import re

TEMPLATES = {
    'common': '''
msgid ""
msgstr ""
"Project-Id-Version: \\n"
"POT-Creation-Date: \\n"
"Last-Translator: \\n"
"Language-Team: \\n"
"Language: \\n"
"MIME-Version: \\n"
"Content-Type: text/plain; charset=utf-8\\n"
"Content-Transfer-Encoding: 8bit\\n"

'''
}

MSGID_REGEX = re.compile(r'^msgid\s+\"(.+?)\"\s*')



import inspect
from typing import Generator


def iter_subclasses(cls: type, max_level: int = -1) -> Generator:
    assert inspect.isclass(cls)
    assert isinstance(max_level, int)

    if max_level == 0:
        return

    for subcls in cls.__subclasses__():
        yield subcls
        for subsubcls in iter_subclasses(subcls, max_level - 1):
            yield subsubcls



from i18n.enums import TranslatableEnum
from i18n.types import BaseTranslatableEnum
from i18n.tools import (
    extract_keys
)
from i18n.settings import (
    VERSION,
    APP,
    AUTHOR
)
from i18n.utils.gettext import (
    set_domain,
    set_language,
    get_language,
    language
)

__name__ = APP
__version__ = VERSION
__author__ = AUTHOR

__all__ = [
    'BaseTranslatableEnum',
    'TranslatableEnum',
    'extract_keys',
    'set_language',
    'get_language',
    'set_domain',
    'language'
]
